@@ -50,6 +50,12 @@ def _fix_time_stamp_format(timestamp):
 
     # convert to UTC
     abstract_date_time = abstract_date_time.astimezone(datetime.timezone.utc)
+
+    microsecond = abstract_date_time.microsecond
+
+    # round off microsecond upto 3 digits
+    abstract_date_time = abstract_date_time.replace(microsecond=round(microsecond, -3))
+
     # normalise precision to ms and convert to ISO string using "Z" instead of +00:00
     fixed = abstract_date_time.isoformat(timespec='milliseconds')[:-6] + "Z"
 
@@ -58,23 +64,25 @@ def _fix_time_stamp_format(timestamp):
 
 
 def _child_to_pre_hash_string(child, sub_child_order):
+    logging.debug("Processing '%s'", child)
     text = ""
     grand_child_text = ""
     if sub_child_order:
         grand_child_text = _recurse_through_children_in_order(child[2], sub_child_order)
     if child[1]:
         text = child[1].strip()
-        if child[0].lower().find("time") > 0 and child[0].lower().find("offset") < 0:
+        if child[0].lower().find("time") >= 0 and child[0].lower().find("offset") < 0:
             text = _fix_time_stamp_format(text)
         else:
             text = _canonize_value(text)
 
         if text:
             text = "=" + text
-            logging.debug("Adding text '%s'", text)
 
     if text or grand_child_text:
-        return child[0] + text + grand_child_text
+        re = child[0] + text + grand_child_text
+        logging.debug("pre hash string element: '%s'", text)
+        return re
 
     return ""
 
@@ -93,6 +101,9 @@ def _recurse_through_children_in_order(child_list, child_order):
     """
     pre_hash = ""
     logging.debug("Calculating pre hash for child list %s \nWith order %s", child_list, child_order)
+
+    user_extensions = _gather_user_extensions(child_list)
+
     for (child_name, sub_child_order) in child_order:
         children = [x for x in child_list if x[0] == child_name]  # elements with the same name
         list_of_values = []
@@ -116,6 +127,10 @@ def _recurse_through_children_in_order(child_list, child_order):
                 list_of_values.insert(0, pre_hash)  # yields correct Joining behavior
             pre_hash = JOIN_BY.join(list_of_values)
 
+    if len(user_extensions) > 0:
+        user_extensions_prehash = _generic_child_list_to_prehash_string(user_extensions)
+        pre_hash = pre_hash + JOIN_BY + user_extensions_prehash
+
     logging.debug("child list pre hash is %s", pre_hash)
 
     return pre_hash
@@ -129,20 +144,49 @@ def _canonize_value(text):
     if converted:
         logging.debug("Converted %s to %s", text, converted)
         return converted
+    logging.debug("No canonical form for '%s'", text)
     return text
 
 
+def _gather_user_extensions(child_list):
+    """
+    Collect user extensions enclosed in child like sensorElementList, readPoint, etc.
+    So that user extensions can be appended to its enclosing element only
+    """
+    user_extensions = []
+
+    if len(child_list) <= 1:
+        return user_extensions
+
+    # ignore top level user extensions
+    for child in child_list:
+        if 'eventTime' in child[0] or 'action' in child[0]:
+            return user_extensions
+
+    # collect user extensions in a separate list
+    for x in child_list:
+        if isinstance(x, tuple) and ('{' in x[0] and '/}' in x[0]):
+            user_extensions.append(x)
+
+    # remove user extensions from original list
+    if user_extensions:
+        for element_to_remove in user_extensions:
+            child_list.remove(element_to_remove)
+
+    return user_extensions
+
+
 def _try_format_web_vocabulary(text):
     """Replace old CBV URNs by new web vocabulary equivalents."""
     return text.replace(
-        'urn:epcglobal:cbv:bizstep:', 'https://ns.gs1.org/voc/Bizstep-'
+        'urn:epcglobal:cbv:bizstep:', 'https://ref.gs1.org/cbv/BizStep-'
     ).replace(
-        'urn:epcglobal:cbv:disp:', 'https://ns.gs1.org/voc/Disp-'
+        'urn:epcglobal:cbv:disp:', 'https://ref.gs1.org/cbv/Disp-'
     ).replace(
-        'urn:epcglobal:cbv:btt:', 'https://ns.gs1.org/voc/BTT-'
+        'urn:epcglobal:cbv:btt:', 'https://ref.gs1.org/cbv/BTT-'
     ).replace(
-        'urn:epcglobal:cbv:sdt:', 'https://ns.gs1.org/voc/SDT-'
-    ).replace('urn:epcglobal:cbv:er:', 'https://ns.gs1.org/voc/ER-')
+        'urn:epcglobal:cbv:sdt:', 'https://ref.gs1.org/cbv/SDT-'
+    ).replace('urn:epcglobal:cbv:er:', 'https://ref.gs1.org/cbv/ER-')
 
 
 def _try_format_numeric(text):
@@ -163,16 +207,32 @@ def _generic_child_list_to_prehash_string(children):
     logging.debug("Parsing remaining elements in: %s", children)
 
     for child in children:
-        text = child[1].strip()
-        if text:
-            text = _canonize_value(text)
-            text = "=" + text
-        list_of_values.append(child[0] + text + _generic_child_list_to_prehash_string(child[2]))
+        if isinstance(child, tuple) and len(child) == 2 and isinstance(child[0], tuple):
+            list_of_values.append(_generic_child_list_to_prehash_string(child))
+        else:
+            text = child[1].strip()
+            if text:
+                text = _canonize_value(text)
+                text = "=" + text
+            list_of_values.append(child[0] + text + _generic_child_list_to_prehash_string(child[2]))
 
-    list_of_values.sort()
+    if len(children) > 1 and should_sort(children):
+        list_of_values.sort()
     return JOIN_BY.join(list_of_values)
 
 
+def should_sort(children):
+    """
+    avoid sort for 'bizTransaction', 'source', 'destination' to match order as defined in CBV 2.0
+    :param children:
+    :return: True/False
+    """
+    for child in children:
+        if child[0] == 'bizTransaction' or child[0] == 'source' or child[0] == 'destination':
+            return False
+    return True
+
+
 def _gather_elements_not_in_order(children, child_order):
     """
     Collects vendor extensions not covered by the defined child order. Consumes the root.
@@ -180,10 +240,8 @@ def _gather_elements_not_in_order(children, child_order):
 
     # remove fields that are to be ignored in the hash:
     # remove all elements from XML tree which do shouldn't take part in hash calculation
-    to_be_ignored = ["recordTime", "eventID"]
-    for child in children:
-        if child[0] in to_be_ignored:
-            children.remove(child)
+    to_be_ignored = ["recordTime", "eventID", "type", "errorDeclaration"]
+    children = [child for child in children if child[0] not in to_be_ignored]
     if children:
         return _generic_child_list_to_prehash_string(children)
 
@@ -233,8 +291,8 @@ def calculate_hashes_from_pre_hashes(prehash_string_list, hashalg="sha256"):
         if hashalg == 'sha256':
             hash_string = 'ni:///sha-256;' + \
                           hashlib.sha256(pre_hash_string.encode('utf-8')).hexdigest() + '?ver=CBV2.0'
-        elif hashalg == 'sha3_256':
-            hash_string = 'ni:///sha3_256;' + \
+        elif hashalg == 'sha3-256':
+            hash_string = 'ni:///sha3-256;' + \
                           hashlib.sha3_256(pre_hash_string.encode('utf-8')).hexdigest() + '?ver=CBV2.0'
         elif hashalg == 'sha384':
             hash_string = 'ni:///sha-384;' + \
